@@ -4,7 +4,9 @@ mod std_impls;
 #[cfg(feature = "uuid")]
 mod uuid;
 
+use std::fmt;
 use std::iter;
+use std::rc::Rc;
 
 pub enum ValueOrQueryable<'a> {
     Value(Value),
@@ -15,9 +17,6 @@ pub struct ValueIter<'a>(pub Box<dyn Iterator<Item = Value> + 'a>);
 pub struct TreeIter<'a>(pub Box<dyn Iterator<Item = &'a dyn Queryable<'a>> + 'a>);
 
 pub trait Queryable<'a> {
-    fn keys(&'a self) -> ValueIter<'a> {
-        ValueIter::empty()
-    }
     fn member<'f>(&'a self, _: &'f Value) -> Option<&'a dyn Queryable<'a>> {
         None
     }
@@ -34,6 +33,49 @@ pub enum Value {
     String(String),
     Int(i64),
     Bool(bool),
+    Array(Array),
+}
+
+#[derive(Debug, Clone, PartialEq, PartialOrd, Eq, Ord)]
+pub struct Array(pub Rc<[Value]>);
+
+impl Array {
+    pub fn len(&self) -> usize {
+        self.0.len()
+    }
+
+    pub fn is_empty(&self) -> bool {
+        self.0.is_empty()
+    }
+}
+
+impl IntoIterator for Array {
+    type Item = Value;
+    type IntoIter = ArrayIter;
+    fn into_iter(self) -> ArrayIter {
+        ArrayIter {
+            current: 0,
+            array: self,
+        }
+    }
+}
+
+pub struct ArrayIter {
+    current: usize,
+    array: Array,
+}
+
+impl Iterator for ArrayIter {
+    type Item = Value;
+    fn next(&mut self) -> Option<Value> {
+        if self.current == self.array.len() {
+            None
+        } else {
+            let index = self.current;
+            self.current += 1;
+            Some(self.array.0[index].clone())
+        }
+    }
 }
 
 impl Value {
@@ -49,6 +91,17 @@ impl Value {
     }
 }
 
+impl fmt::Display for Value {
+    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        match self {
+            Value::String(v) => write!(f, "{}", v),
+            Value::Int(v) => write!(f, "{}", v),
+            Value::Bool(v) => write!(f, "{}", v),
+            Value::Array(v) => write!(f, "{:?}", v),
+        }
+    }
+}
+
 impl<'a> ValueIter<'a> {
     pub fn empty() -> Self {
         ValueIter(Box::new(iter::empty()))
@@ -57,12 +110,26 @@ impl<'a> ValueIter<'a> {
     pub fn once<V: Into<Value>>(val: V) -> Self {
         ValueIter(Box::new(iter::once(val.into())))
     }
+
+    pub fn from_values<T: 'a>(values: impl IntoIterator<Item = T> + 'a) -> Self
+    where
+        Value: From<T>,
+    {
+        ValueIter(Box::from(values.into_iter().map(Value::from)))
+    }
 }
 
 impl<'a> TreeIter<'a> {
     pub fn empty() -> Self {
         TreeIter(Box::new(iter::empty()))
     }
+
+    pub fn from_queryables<Q>(queryables: impl IntoIterator<Item = &'a Q> + 'a) -> Self
+    where
+        Q: Queryable<'a> + 'a,
+    {
+        TreeIter(Box::from(queryables.into_iter().map(|q| q as _)))
+    }
 }
 
 impl<'a> Iterator for ValueIter<'a> {
@@ -86,13 +153,6 @@ impl<'a> Iterator for TreeIter<'a> {
 }
 
 impl<'a> Queryable<'a> for ValueOrQueryable<'a> {
-    fn keys(&'a self) -> ValueIter<'a> {
-        if let ValueOrQueryable::Queryable(q) = self {
-            q.keys()
-        } else {
-            ValueIter::empty()
-        }
-    }
     fn member<'f>(&'a self, field: &'f Value) -> Option<&'a dyn Queryable<'a>> {
         if let ValueOrQueryable::Queryable(q) = self {
             q.member(field)
